"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CypherAccount = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../utils");
const mango_client_1 = require("@blockworks-foundation/mango-client");
const pda_1 = require("../utils/pda");
const ordersAccount_1 = require("./ordersAccount");
const associated_token_1 = require("@project-serum/associated-token");
class CypherAccount {
    constructor(client, address, state, _onStateUpdate) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, clearing, authority, accountNumber = 0) {
        const [account, bump] = (0, utils_1.deriveAccountAddress)(authority, accountNumber, client.cypherPID);
        const ix = await client.methods
            .createAccount(accountNumber, bump)
            .accountsStrict({
            clearing,
            masterAccount: account,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            account,
            ixs: [ix],
            signers: []
        };
    }
    static async load(client, address, onStateUpdateHandler) {
        const state = await client.accounts.cypherAccount.fetchNullable(address);
        if (state === null)
            return null;
        return new CypherAccount(client, address, state, onStateUpdateHandler);
    }
    getSubAccounts() {
        const subAccounts = [];
        for (const subAccountCache of this.state.subAccountCaches) {
            if (!subAccountCache.subAccount.equals(web3_js_1.PublicKey.default)) {
                subAccounts.push(subAccountCache.subAccount);
            }
        }
        return subAccounts;
    }
    getCRatio() {
        const totalAssetsValue = mango_client_1.I80F48.fromNumber(0);
        const totalLiabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        for (const subAccountCache of this.state.subAccountCaches) {
            if (!subAccountCache.subAccount.equals(web3_js_1.PublicKey.default)) {
                const assetsValue = new mango_client_1.I80F48(subAccountCache.assetsValue);
                const liabsValue = new mango_client_1.I80F48(subAccountCache.liabilitiesValue);
                if (subAccountCache.margining.cross) {
                    totalAssetsValue.iadd(assetsValue);
                    totalLiabilitiesValue.iadd(liabsValue);
                }
                // const cRatio = new I80F48(subAccountCache.cRatio);
                // console.log(
                //   'SubAccount:',
                //   subAccountCache.subAccount.toString(),
                //   'Assets:',
                //   assetsValue.toFixed(4),
                //   'Liabilities:',
                //   liabsValue.toFixed(4),
                //   'C-Ratio:',
                //   cRatio.toFixed(4)
                // );
            }
        }
        return {
            assetsValue: totalAssetsValue,
            liabilitiesValue: totalLiabilitiesValue,
            cRatio: totalLiabilitiesValue.isZero()
                ? new mango_client_1.I80F48(mango_client_1.I80F48.MAX_BN)
                : totalAssetsValue.div(totalLiabilitiesValue)
        };
    }
    subscribe() {
        this.client.accounts.cypherAccount
            .subscribe(this.address)
            .on('change', (state) => {
            this.state = state;
            // todo: check if dexMarkets need to be reloaded.(market listing/delisting)
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        });
    }
    async unsubscribe() {
        await this.client.accounts.cypherAccount.unsubscribe(this.address);
    }
    getSpotOpenOrdersAddress(dexMarket, subAccount) {
        const [address] = (0, pda_1.deriveSerumOrdersAccountAddress)(dexMarket, this.address, subAccount, this.client.cypherPID);
        return address;
    }
    getDerivativesOpenOrdersAddress(market) {
        const [address] = (0, pda_1.deriveOrdersAccountAddress)(this.address, market, this.client.cypherPID);
        return address;
    }
    async hasSpotOpenOrders(dexMarket, subAccount) {
        const spotOpenOrders = this.getSpotOpenOrdersAddress(dexMarket, subAccount);
        const accountState = await this.client.connection.getAccountInfo(spotOpenOrders);
        if (accountState) {
            return true;
        }
        else {
            return false;
        }
    }
    async hasDerivativeOpenOrders(market) {
        const ordersAccountAddress = this.getDerivativesOpenOrdersAddress(market);
        const accountState = await this.client.accounts.ordersAccount.fetchNullable(ordersAccountAddress);
        if (accountState) {
            return true;
        }
        else {
            return false;
        }
    }
    async getSpotMarketOrders(pool) {
        if (pool.market)
            return await pool.market.loadOrdersForOwner(this.client.connection, this.address);
        return [];
    }
    async getDerivativeMarketOrders(market) {
        const ordersAccountAddress = this.getDerivativesOpenOrdersAddress(market.address);
        const ordersAccount = await ordersAccount_1.DerivativesOrdersAccount.load(this.client, ordersAccountAddress);
        return ordersAccount.getOrders(market);
    }
    async getAssociatedTokenAddress(tokenMint) {
        return (0, associated_token_1.getAssociatedTokenAddress)(this.client.walletPubkey, tokenMint);
    }
}
exports.CypherAccount = CypherAccount;
//# sourceMappingURL=account.js.map