"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CypherSubAccount = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../utils");
const mango_client_1 = require("@blockworks-foundation/mango-client");
const spotPosition_1 = require("../viewers/spotPosition");
const derivativePosition_1 = require("../viewers/derivativePosition");
class CypherSubAccount {
    constructor(client, address, state, _onStateUpdate) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, authority, masterAccount, accountNumber = 0, accountAlias = Array(32).fill(0)) {
        const [subAccount, bump] = (0, utils_1.deriveSubAccountAddress)(masterAccount, accountNumber, client.cypherPID);
        const ix = await client.methods
            .createSubAccount(accountNumber, bump, accountAlias)
            .accountsStrict({
            masterAccount,
            subAccount,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            subAccount,
            ixs: [ix],
            signers: []
        };
    }
    static async load(client, address, onStateUpdateHandler) {
        const state = (await client.accounts.cypherSubAccount.fetchNullable(address));
        return new CypherSubAccount(client, address, state, onStateUpdateHandler);
    }
    getSpotPosition(tokenMint) {
        for (const position of this.state.positions) {
            if (position.spot.tokenMint.equals(tokenMint)) {
                return new spotPosition_1.SpotPosition(position.spot);
            }
        }
        return null;
    }
    // The position does not account for deposit or borrow interest
    // To get the actual value of the position
    // If its a deposit, multiply it by the deposit interest rate
    // If its a borrow, multiply it by the borrow interest rate
    // The index can be found in the resp cypher pool
    getSpotPositionsInfo() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                positions.push(position.spot);
            }
        }
        return positions;
    }
    getSpotPositions() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                positions.push(new spotPosition_1.SpotPosition(position.spot));
            }
        }
        return positions;
    }
    getDerivativePosition(market) {
        for (const position of this.state.positions) {
            if (position.derivative.market.equals(market)) {
                return new derivativePosition_1.DerivativePosition(position.derivative);
            }
        }
        return null;
    }
    getDerivativePositionsInfo() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.derivative.market.equals(web3_js_1.PublicKey.default)) {
                positions.push(position.derivative);
            }
        }
        return positions;
    }
    getDerivativePositions() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.derivative.market.equals(web3_js_1.PublicKey.default)) {
                positions.push(new derivativePosition_1.DerivativePosition(position.derivative));
            }
        }
        return positions;
    }
    getAssetsValue(cacheAccount) {
        const assetsValue = mango_client_1.I80F48.fromNumber(0);
        for (const { spot, derivative } of this.state.positions) {
            if (!spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(spot.cacheIndex);
                const openOrdersCache = spot.openOrdersCache;
                const oraclePrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                const weight = mango_client_1.I80F48.fromNumber(priceCache.spotInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                const basePosition = new mango_client_1.I80F48(spot.position);
                if (basePosition.isPos()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition, priceCache.decimals)
                        .mul(oraclePrice)
                        .mul(weight);
                    assetsValue.iadd(positionValue);
                    // console.log(
                    //   'Asset ----- Token: ',
                    //   spot.tokenMint.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, priceCache.decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   oraclePrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
                if (openOrdersCache.coinTotal != mango_client_1.ZERO_BN) {
                    assetsValue.iadd(mango_client_1.I80F48.fromU64(openOrdersCache.coinTotal)
                        .mul(oraclePrice)
                        .mul(weight));
                }
                assetsValue.iadd(mango_client_1.I80F48.fromU64(openOrdersCache.pcTotal));
            }
            if (!derivative.market.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(derivative.cacheIndex);
                const openOrdersCache = derivative.openOrdersCache;
                let decimals = 0;
                if (derivative.marketType.perpetualFuture) {
                    decimals = priceCache.perpDecimals;
                }
                else {
                    decimals = priceCache.futuresDecimals;
                }
                let derivPrice = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    derivPrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                }
                else {
                    derivPrice = new mango_client_1.I80F48(priceCache.marketPrice);
                }
                let weight = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.perpInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                else {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.futuresInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                const basePosition = new mango_client_1.I80F48(derivative.basePosition);
                if (basePosition.isPos()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition, decimals)
                        .mul(derivPrice)
                        .mul(weight);
                    assetsValue.iadd(positionValue);
                    // console.log(
                    //   'Asset ----- Market: ',
                    //   derivative.market.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   derivPrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
                if (openOrdersCache.coinTotal != mango_client_1.ZERO_BN) {
                    assetsValue.iadd(mango_client_1.I80F48.fromU64(openOrdersCache.coinTotal)
                        .mul(derivPrice)
                        .mul(weight));
                }
                assetsValue.iadd(mango_client_1.I80F48.fromU64(openOrdersCache.pcTotal));
            }
        }
        return assetsValue;
    }
    getLiabilitiesValue(cacheAccount) {
        const liabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        for (const { spot, derivative } of this.state.positions) {
            if (!spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(spot.cacheIndex);
                const oraclePrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                const weight = mango_client_1.I80F48.fromNumber(priceCache.spotInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                const basePosition = new mango_client_1.I80F48(spot.position);
                if (basePosition.isNeg()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition.abs(), priceCache.decimals)
                        .mul(oraclePrice)
                        .mul(weight);
                    liabilitiesValue.iadd(positionValue);
                    // console.log(
                    //   'Liability ----- Token: ',
                    //   spot.tokenMint.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, priceCache.decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   oraclePrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
            }
            if (!derivative.market.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(derivative.cacheIndex);
                let decimals = 0;
                if (derivative.marketType.perpetualFuture) {
                    decimals = priceCache.perpDecimals;
                }
                else {
                    decimals = priceCache.futuresDecimals;
                }
                let derivPrice = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    derivPrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                }
                else {
                    derivPrice = new mango_client_1.I80F48(priceCache.marketPrice);
                }
                let weight = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.perpInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                else {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.futuresInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                const basePosition = new mango_client_1.I80F48(derivative.basePosition);
                if (basePosition.isNeg()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition.abs(), decimals)
                        .mul(derivPrice)
                        .mul(weight);
                    liabilitiesValue.iadd(positionValue);
                    // console.log(
                    //   'Liability ----- Market: ',
                    //   derivative.market.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   derivPrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
            }
        }
        return liabilitiesValue;
    }
    subscribe() {
        this.client.accounts.cypherSubAccount
            .subscribe(this.address)
            .on('change', (state) => {
            this.state = state;
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        });
    }
    async unsubscribe() {
        await this.client.accounts.cypherSubAccount.unsubscribe(this.address);
    }
}
exports.CypherSubAccount = CypherSubAccount;
//# sourceMappingURL=subAccount.js.map