"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FaucetClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const anchor_1 = require("@project-serum/anchor");
const associated_token_1 = require("@project-serum/associated-token");
const instructions_1 = require("../instructions");
const constants_1 = require("../constants");
const faucet_json_1 = __importDefault(require("../generated/idl/faucet.json"));
const faucetMint_json_1 = __importDefault(require("./faucetMint.json"));
class FaucetClient {
    constructor(_cluster, rpcEndpoint, wallet, confirmOpts = anchor_1.AnchorProvider.defaultOptions()) {
        this._cluster = _cluster;
        const conn = new web3_js_1.Connection(rpcEndpoint, confirmOpts.commitment);
        const provider = new anchor_1.AnchorProvider(conn, wallet, confirmOpts);
        this._program = new anchor_1.Program(faucet_json_1.default, constants_1.CONFIGS[_cluster].FAUCET_PID, provider);
    }
    static get quoteMintAddrFull() {
        return web3_js_1.Keypair.fromSecretKey(Uint8Array.from(faucetMint_json_1.default));
    }
    static deriveFaucetAddress(cluster, mint) {
        return anchor_1.utils.publicKey.findProgramAddressSync([anchor_1.utils.bytes.utf8.encode('FAUCET'), mint.toBuffer()], constants_1.CONFIGS[cluster].FAUCET_PID)[0];
    }
    static deriveMintAuthority(cluster, mint) {
        return anchor_1.utils.publicKey.findProgramAddressSync([anchor_1.utils.bytes.utf8.encode('MINT_AUTHORITY'), mint.toBuffer()], constants_1.CONFIGS[cluster].FAUCET_PID);
    }
    get _provider() {
        return this._program?.provider;
    }
    get _walletPubkey() {
        return this._provider?.wallet.publicKey;
    }
    async initFaucet(mint, decimals) {
        const faucet = FaucetClient.deriveFaucetAddress(this._cluster, mint.publicKey);
        const [mintAuthority, mintAuthorityBump] = FaucetClient.deriveMintAuthority(this._cluster, mint.publicKey);
        return this._program.methods
            .initFaucet(mintAuthorityBump)
            .accounts({
            faucet: faucet,
            mint: mint.publicKey,
            mintAuthority: mintAuthority,
            payer: this._walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .preInstructions(await (0, instructions_1.makeCreateMintIxs)(this._provider, mintAuthority, mint.publicKey, decimals))
            .signers([mint])
            .rpc();
    }
    async getRequestInstr(mint, target) {
        const faucet = FaucetClient.deriveFaucetAddress(this._cluster, mint);
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [mintAuthority, mintBump] = FaucetClient.deriveMintAuthority(this._cluster, mint);
        return this._program.methods
            .request()
            .accounts({
            faucet,
            mint,
            target,
            mintAuthority,
            tokenProgram: spl_token_1.TOKEN_PROGRAM_ID
        })
            .instruction();
    }
    async mint(mint) {
        const associatedTokenAddr = await (0, associated_token_1.getAssociatedTokenAddress)(this._walletPubkey, mint);
        const info = await this._program.provider.connection.getAccountInfo(associatedTokenAddr, this._program.provider.connection.commitment);
        const tx = new web3_js_1.Transaction();
        if (info === null) {
            tx.add(await (0, associated_token_1.createAssociatedTokenAccount)(this._walletPubkey, this._walletPubkey, mint));
        }
        tx.add(await this.getRequestInstr(mint, associatedTokenAddr));
        return this._provider.sendAndConfirm(tx);
    }
}
exports.FaucetClient = FaucetClient;
//# sourceMappingURL=faucet.js.map